@@ -22,7 +22,6 @@ import operator
 
 from automaton import exceptions as automaton_errors
 from ironicclient import exceptions
-from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_db.sqlalchemy import utils as db_utils
 from oslo_utils import excutils
@@ -34,26 +33,15 @@ from sqlalchemy.orm import exc as orm_errors
 
 from ironic_inspector.common.i18n import _
 from ironic_inspector.common import ironic as ir_utils
+from ironic_inspector.common import locking
 from ironic_inspector import db
 from ironic_inspector import introspection_state as istate
 from ironic_inspector import utils
 
 
 CONF = cfg.CONF
-
-
 LOG = utils.getProcessingLogger(__name__)
-
-
 MACS_ATTRIBUTE = 'mac'
-_LOCK_TEMPLATE = 'node-%s'
-_SEMAPHORES = lockutils.Semaphores()
-
-
-def _get_lock(uuid):
-    """Get lock object for a given node UUID."""
-    return lockutils.internal_lock(_LOCK_TEMPLATE % uuid,
-                                   semaphores=_SEMAPHORES)
 
 
 class NodeInfo(object):
@@ -84,13 +72,12 @@ class NodeInfo(object):
         # equivalent to True actually.
         self._manage_boot = manage_boot if manage_boot is not None else True
         # This is a lock on a node UUID, not on a NodeInfo object
-        self._lock = _get_lock(uuid)
+        self._lock = locking.get_lock(uuid)
         # Whether lock was acquired using this NodeInfo object
-        self._locked = False
         self._fsm = None
 
     def __del__(self):
-        if self._locked:
+        if self._lock.is_locked():
             LOG.warning('BUG: node lock was not released by the moment '
                         'node info object is deleted')
             self._lock.release()
@@ -112,12 +99,11 @@ class NodeInfo(object):
                          return immediately.
         :returns: boolean value, whether lock was acquired successfully
         """
-        if self._locked:
+        if self._lock.is_locked():
             return True
 
         LOG.debug('Attempting to acquire lock', node_info=self)
         if self._lock.acquire(blocking):
-            self._locked = True
             LOG.debug('Successfully acquired lock', node_info=self)
             return True
         else:
@@ -129,10 +115,9 @@ class NodeInfo(object):
 
         Does nothing if lock was not acquired using this NodeInfo object.
         """
-        if self._locked:
+        if self._lock.is_locked():
             LOG.debug('Successfully released lock', node_info=self)
             self._lock.release()
-        self._locked = False
 
     @property
     def version_id(self):
@@ -663,7 +648,7 @@ def release_lock(func):
         finally:
             # FIXME(milan) hacking the test cases to work
             # with release_lock.assert_called_once...
-            if node_info._locked:
+            if node_info._lock.is_locked():
                 node_info.release_lock()
     return inner
 
@@ -738,7 +723,7 @@ def delete_nodes_not_in_list(uuids):
     for uuid in inspector_uuids - uuids:
         LOG.warning('Node %s was deleted from Ironic, dropping from Ironic '
                     'Inspector database', uuid)
-        with _get_lock(uuid):
+        with locking.get_lock(uuid):
             _delete_node(uuid)
 
 
@@ -926,7 +911,7 @@ def create_node(driver, ironic=None, **attributes):
     * Sets node_info state to enrolling.
 
     :param driver: driver for Ironic node.
-    :param ironic: ronic client instance.
+    :param ironic: ironic client instance.
     :param attributes: dict, additional keyword arguments to pass
                              to the ironic client on node creation.
     :return: NodeInfo, or None in case error happened.
@@ -942,6 +927,44 @@ def create_node(driver, ironic=None, **attributes):
         return add_node(node.uuid, istate.States.enrolling, ironic=ironic)
 
 
+def record_node(ironic=None, bmc_addresses=None, macs=None):
+    """Create a cache record for a known active node.
+
+    :param ironic: ironic client instance.
+    :param bmc_addresses: list of BMC addresses.
+    :param macs: list of MAC addresses.
+    :return: NodeInfo
+    """
+    if not bmc_addresses and not macs:
+        raise utils.NotFoundInCacheError(
+            _("Existing node cannot be found since neither MAC addresses "
+              "nor BMC addresses are present in the inventory"))
+
+    if ironic is None:
+        ironic = ir_utils.get_client()
+
+    node = ir_utils.lookup_node(macs=macs, bmc_addresses=bmc_addresses,
+                                ironic=ironic)
+    if not node:
+        bmc_addresses = ', '.join(bmc_addresses) if bmc_addresses else None
+        macs = ', '.join(macs) if macs else None
+        raise utils.NotFoundInCacheError(
+            _("Existing node was not found by MAC address(es) %(macs)s "
+              "and BMC address(es) %(addr)s") %
+            {'macs': macs, 'addr': bmc_addresses})
+
+    node = ironic.node.get(node, fields=['uuid', 'provision_state'])
+    # TODO(dtantsur): do we want to allow updates in all states?
+    if node.provision_state not in ir_utils.VALID_ACTIVE_STATES:
+        raise utils.Error(_("Node %(node)s is not active, its provision "
+                            "state is %(state)s") %
+                          {'node': node.uuid,
+                           'state': node.provision_state})
+
+    return add_node(node.uuid, istate.States.waiting,
+                    manage_boot=False, mac=macs, bmc_address=bmc_addresses)
+
+
 def get_node_list(ironic=None, marker=None, limit=None):
     """Get node list from the cache.
 
